// **** BEGIN LICENSE BLOCK ****
// Copyright(c) 2008 Edoardo Gargano
//
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// Portions of this code have been based upon
// LiveHttpHeaders - http://livehttpheaders.mozdev.org
// Copyright(c) 2002-2003 Daniel Savard.
// And TamperData - http://tamperdata.mozdev.org
// Copyright(c) 2005 Adam Judson
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//
//
// http.js - track and modify http requests and responses
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA
//
// **** END LICENSE BLOCK ****


		
var oTamper;
function startTamper() {
   oTamper = new Tamper();
   oTamper.start();
   SegodocUtils.log("startTamper");
}

function stopTamper() {
   if(oTamper!=undefined) {
   oTamper.stop();
   delete oTamper;
   oTamper = null;
   SegodocUtils.log("stopTamper");
   }

}

function Tamper() {
}

Tamper.POST_DATA = "POSTDATA";

Tamper.prototype = { 
  
   forceCaching: false,
   tamper : false,
   started: false,

   set isTampering(value) {
      this.tamper = value;
      segodoc.toggleTampering(this.tamper);
   },
   get isTampering() {
      return this.tamper;
   },
   // Initialisation and termination functions
   start : function() {
   	SegodocUtils.log(" start ");
      this.requests = new Array();
      this.started = true;
      this.addToListener();
   },
   
   stop : function() {
   	SegodocUtils.log(" stop ");
      this.removeFromListener();
      this.list = null;
      this.requests = null;
      this.started = false;
   },

   startTampering : function() {
      this.isTampering = true;
   },

   stopTampering : function() {
      this.isTampering = false;
   },

   // This is the observerService's observe listener.
   observe: function(aSubject, aTopic, aData) {
   	aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
   	var uri = aSubject.URI.asciiSpec;
 	if(segodoc.checkCrypted(uri)) {
      if (aTopic == 'http-on-modify-request') {
          this.onModifyRequest(aSubject);
      } else if (aTopic == 'http-on-examine-response') {
         // #nsITraceableChannel from FF 3.0.4#
      	 var newListener = new TracingListener(); // nsITraceableChannel
         try {
	         aSubject.QueryInterface(Components.interfaces.nsITraceableChannel);
	         newListener.originalListener = aSubject.setNewListener(newListener);
         } catch(e) {
         	alert("Prolem with nsITraceableChannel");
         }         
      }
 	} else {
		// SegodocUtils.log("Not a Segodoc document");
 	}// closing checkCrypted
   },
   
   onModifyRequest : function (oHttp) {
      var uri = oHttp.URI.asciiSpec;

      // make sure we are caching
      this.forceCaching(oHttp);
      var loadFlags = this.getStringVersionOfLoadFlags(oHttp.loadFlags);
      var loadFromCache = this.isLoadFromCache(oHttp.loadFlags);
      var cancelled = false;
      if (!loadFromCache) {
         this.modifyForReplay(uri, oHttp);
      }
      // Get the request headers
      var visitor = new HeaderInfoVisitor(oHttp);
      var requestHeaders = visitor.visitRequest();
      var postData = visitor.getPostData();
      var postBodyHeaders = visitor.getPostBodyHeaders();
      var requestResponse = new RequestResponse(); 
      // SegodocUtils.log(postBodyHeaders.toString());
      requestResponse.setRequestData(uri, new Date(), oHttp.requestMethod, requestHeaders, postBodyHeaders, postData, loadFlags);
      if (cancelled) {
         // we cancelled it
         // which really does nothing...
         requestResponse.cancel();
      } else if (loadFromCache) {
         requestResponse.setLoadFromCache();
      }
      this.addRow(requestResponse);
   },

   modifyForReplay : function(uri, request) {
      if (this.replayList && (uri in this.replayList)) {
         var item = this.replayList[uri];
         delete this.replayList[uri];
         var headers = item.requestHeaders;

         this.removeAllHeaders(request);

         // and replace them with the old copy
         for (var oldHeader in headers) {
            request.setRequestHeader(oldHeader, headers[oldHeader], null);
         }
      }
      startCrypt(request, uri);
   },

   removeAllHeaders : function(request) {
      function emptyObserver(request) {
         this.oHttp = request;
         this.request = new Array();
      }
      emptyObserver.prototype = {
         visitHeader : function (name, value) {
            this.request[name] = value;
         },
   
         emptyHeaders: function () {
            this.oHttp.visitRequestHeaders(this);
            for (var i in this.request) {
               this.oHttp.setRequestHeader(i, null, false);
            }
         }
      };
      var empty = new emptyObserver(request);
      empty.emptyHeaders();
   },

   addToReplayList : function(uri, headerArray) {
      if (!this.replayList) {
         this.replayList = {};
      }
      this.replayList[uri] = headerArray;
   },

   addRow : function(requestResponse) {
      this.requests.push(requestResponse);
   },

   // most of these never get hit
   getStringVersionOfLoadFlags : function(flags) {
      var flagString = "";
      if (flags & Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE) {
         flagString += "LOAD_BYPASS_CACHE  ";
      }
      if (flags & Components.interfaces.nsIRequest.LOAD_BACKGROUND) {
         flagString += "LOAD_BACKGROUND  ";
      }
      if (flags & Components.interfaces.nsIRequest.INHIBIT_CACHING) {
         flagString += "INHIBIT_CACHING  ";
      }
      if (flags & Components.interfaces.nsIRequest.INHIBIT_PERSISTENT_CACHING) {
         flagString += "INHIBIT_PERSISTENT_CACHING  ";
      }
      if (flags & Components.interfaces.nsICachingChannel.LOAD_BYPASS_LOCAL_CACHE) {
         flagString += "LOAD_BYPASS_LOCAL_CACHE  ";
      }
      if (flags & Components.interfaces.nsICachingChannel.LOAD_ONLY_FROM_CACHE) {
         flagString += "LOAD_ONLY_FROM_CACHE  ";
      }
      if (flags & Components.interfaces.nsICachingChannel.LOAD_ONLY_IF_MODIFIED) {
         flagString += "LOAD_ONLY_IF_MODIFIED  ";
      }
      if (flags & Components.interfaces.nsIRequest.LOAD_FROM_CACHE) {
         flagString += "LOAD_FROM_CACHE  ";
      }
      if (flags & Components.interfaces.nsIRequest.VALIDATE_ALWAYS) {
         flagString += "VALIDATE_ALWAYS ";
      }
      if (flags & Components.interfaces.nsIRequest.VALIDATE_NEVER) {
         flagString += "VALIDATE_NEVER  ";
      }
      if (flags & Components.interfaces.nsIRequest.VALIDATE_ONCE_PER_SESSION) {
         flagString += "VALIDATE_ONCE_PER_SESSION ";
      }
      if (flags & Components.interfaces.nsIChannel.LOAD_DOCUMENT_URI) {
         flagString += "LOAD_DOCUMENT_URI  ";
      }
      if (flags & Components.interfaces.nsIChannel.LOAD_RETARGETED_DOCUMENT_URI) {
         flagString += "LOAD_RETARGETED_DOCUMENT_URI  ";
      }
      if (flags & Components.interfaces.nsIChannel.LOAD_REPLACE) {
         flagString += "LOAD_REPLACE  ";
      }
      if (flags & Components.interfaces.nsIChannel.LOAD_INITIAL_DOCUMENT_URI) {
         flagString += "LOAD_INITIAL_DOCUMENT_URI  ";
      }
      if (flags & Components.interfaces.nsIChannel.LOAD_TARGETED) {
         flagString += "LOAD_TARGETED  ";
      }
      if ((Components.interfaces.nsICachingChannel.LOAD_BYPASS_LOCAL_CACHE_IF_BUSY) && 
          (flags & Components.interfaces.nsICachingChannel.LOAD_BYPASS_LOCAL_CACHE_IF_BUSY)) {
         flagString += "LOAD_BYPASS_LOCAL_CACHE_IF_BUSY ";
      }
      if (!flagString.length) {
         flagString = "LOAD_NORMAL";
      }
      return flagString;
   },

   forceCaching : function(request) {
      // we only care if we were a POST, GET's cache no matter what
      if (request.requestMethod == "POST") {
         if (request.loadFlags & Components.interfaces.nsIRequest.INHIBIT_CACHING) {
            if (this.forceCaching) {
               request.loadFlags = request.loadFlags & ~Components.interfaces.nsIRequest.INHIBIT_CACHING;
               SegodocUtils.log("Forcing cache on request: [" + request.URI.asciiSpec + "]");
            }
         }
      }
   },   

   isLoadFromCache : function(flags) {
      return flags & Components.interfaces.nsIRequest.LOAD_FROM_CACHE;
   },
   
   QueryInterface: function(iid) {
    if (!iid.equals(Components.interfaces.nsISupports) &&
        !iid.equals(Components.interfaces.nsIHttpNotify) &&
        !iid.equals(Components.interfaces.nsIObserver)) {
          throw Components.results.NS_ERROR_NO_INTERFACE;
      }
      return this;
    },

   addToListener: function() {
      // Register new request and response listener
      // Should be a new version of Mozilla/Phoenix (after september 15, 2003)
      var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      observerService.addObserver(this, "http-on-modify-request",   false);
      observerService.addObserver(this, "http-on-examine-response", false);
   },
   
   removeFromListener: function() {
      // Unregistering listener
      var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      observerService.removeObserver(this, "http-on-examine-response");
      observerService.removeObserver(this, "http-on-modify-request");
   }
};

function HeaderInfoVisitor (oHttp) {
  this.oHttp = oHttp;
  this.headers = new Array();
}

HeaderInfoVisitor.prototype =  {
   extractPostData : function(visitor, oHttp) {
      function postData(stream) {
         // Scriptable Stream Constants
         this.seekablestream = stream;
         this.stream = SegodocUtils.createScriptableInputStream(this.seekablestream);
         
         // Check if the stream has headers
         this.hasheaders = false;
         this.body = 0;
         this.isBinary = true;
         if (this.seekablestream instanceof Components.interfaces.nsIMIMEInputStream) {
            this.seekablestream.QueryInterface(Components.interfaces.nsIMIMEInputStream);
            this.hasheaders = true;
            this.body = -1; // Must read header to find body
            this.isBinary = false;
         } else if (this.seekablestream instanceof Components.interfaces.nsIStringInputStream) {
            this.seekablestream.QueryInterface(Components.interfaces.nsIStringInputStream);
            this.hasheaders = true;
            this.body = -1; // Must read header to find body
         }
      }

      postData.prototype = {
         rewind: function() {
            this.seekablestream.seek(0,0);
         },

         tell: function() {
            return this.seekablestream.tell();
         },
   
         readLine: function() {
            var line = "";
            var size = this.stream.available();
            for (var i = 0; i < size; i++) {
               var c = this.stream.read(1);
               if (c == '\r') {
               } else if (c == '\n') {
                  break;
               } else {
                  line += c;
               }
            }
            return line;
         },
   
         // visitor can be null, function has side-effect of setting body
         visitPostHeaders: function(visitor) {
            if (this.hasheaders) {
               this.rewind();
               var line = this.readLine();
               while(line) {
                  if (visitor) {
                     // SegodocUtils.log("Got a post header: [" + line +
						// "]");
                     var tmp = line.match(/^([^:]+):\s?(.*)/);
                     // match can return null...
                     if (tmp) {
                        visitor.visitPostHeader(tmp[1], tmp[2]);
                        // if we get a tricky content type, then we are binary
                        // e.g. Content-Type=multipart/form-data;
						// boundary=---------------------------41184676334
                        if (!this.isBinary && tmp[1].toLowerCase() == "content-type" && tmp[2].indexOf("multipart") != "-1") {
                           this.isBinary = true;
                        }
                     } else {
                        visitor.visitPostHeader(line, "");
                     }
                  }
                  line = this.readLine();
               }
               this.body = this.tell();
            }
         },
   
         getPostBody: function(visitor) {
            // Position the stream to the start of the body
            if (this.body < 0 || this.seekablestream.tell() != this.body) {
               this.visitPostHeaders(visitor);
            }
            
            var size = this.stream.available();
            if (size == 0 && this.body != 0) {
               // whoops, there weren't really headers..
               this.rewind();
               visitor.clearPostHeaders();
               this.hasheaders = false;
               this.isBinary   = false;
               size = this.stream.available();
            }
            var postString = "";
            try {
               // This is to avoid 'NS_BASE_STREAM_CLOSED' exception that may
				// occurs
               // See bug #188328.
               for (var i = 0; i < size; i++) {
                  var c = this.stream.read(1);
                  c ? postString += c : postString+='\0';
               }
            } catch (ex) {
               return "" + ex;
            } finally {
               this.rewind();
               // this.stream.close();
            }
            // strip off trailing \r\n's
            while (postString.indexOf("\r\n") == (postString.length - 2)) {
               postString = postString.substring(0, postString.length - 2);
            }
            return postString;
         }
      };
   
      // Get the postData stream from the Http Object
      try {
         // Must change HttpChannel to UploadChannel to be able to access
			// post data
         oHttp.QueryInterface(Components.interfaces.nsIUploadChannel);
         // Get the post data stream
         if (oHttp.uploadStream) {
            // Must change to SeekableStream to be able to rewind
            oHttp.uploadStream.QueryInterface(Components.interfaces.nsISeekableStream);
            // And return a postData object
            return new postData(oHttp.uploadStream);
         } 
      } catch (e) {
         SegodocUtils.log("Got an exception retrieving the post data: [" + e + "]");
         return "crap";
      }
      return null;
   },
   
   visitHeader : function(name, value) {
      this.headers[name] = value;
   },

   visitPostHeader : function(name, value) {
      if (!this.postBodyHeaders) {
         this.postBodyHeaders = {};
      }
      this.postBodyHeaders[name] = value;
   },

   clearPostHeaders : function() {
      if (this.postBodyHeaders) {
         delete this.postBodyHeaders;
      }
   },

   visitRequest : function () {
      this.headers = {};
      this.oHttp.visitRequestHeaders(this);
      
      // There may be post data in the request
      var postData = this.extractPostData(this, this.oHttp);
      if (postData) {
         var postBody = postData.getPostBody(this);
         if (postBody !== null) {
            this.postBody = {body : postBody, binary : postData.isBinary};
         }
      }
      return this.headers;
   },

   getPostData : function() {
      return this.postBody ? this.postBody : null;
   },

   getPostBodyHeaders : function() {
      return this.postBodyHeaders ? this.postBodyHeaders : null;
   }

};
// ******************************************************************************

// Copy response listener implementation.
function TracingListener() {
}

TracingListener.prototype =
{
    originalListener: null,
    receivedData: [],   // array for incoming data.
    i: null,
    isPending: false,

    onDataAvailable: function(request, context, inputStream, offset, count)
    {
		this.i++;

        var binaryInputStream = Components.classes["@mozilla.org/binaryinputstream;1"]
        	.createInstance(Components.interfaces.nsIBinaryInputStream);
        var storageStream = Components.classes["@mozilla.org/storagestream;1"]
            .createInstance(Components.interfaces.nsIStorageStream);
        var binaryOutputStream = Components.classes["@mozilla.org/binaryoutputstream;1"]
            .createInstance(Components.interfaces.nsIBinaryOutputStream); 

        binaryInputStream.setInputStream(inputStream);
        storageStream.init(16384, count, null);

        binaryOutputStream.setOutputStream(storageStream.getOutputStream(0));
		
        // Copy received data as they come.
        var data = binaryInputStream.readBytes(count);
//       var responseSource = this.receivedData.join('');
         if(request.URI.asciiSpec.indexOf("docs.google.com")!=-1) {
		   if(data.indexOf("123456789crypted")!=-1 && data.indexOf("987654321crypted")!=-1) {
				var beginCrypted = data.indexOf("123456789crypted");
				var endCrypted = data.indexOf("987654321crypted");
				var crypted = data.substring(beginCrypted + 16, endCrypted);
				// call decrypt
				var decrypted = segodoc.decrypt(crypted,request.URI.asciiSpec);
				var firstPart = data.substring(0, beginCrypted);
				var secondPart = data.substring(endCrypted + 16, data.length)
				data = firstPart + decrypted + secondPart;
				
		   } else if(data.indexOf("987654321crypted")!=-1) {
		  		SegodocUtils.log("FRAG END: "+data);
		     try {
		   		var endCrypted = data.indexOf("987654321crypted");
		   		var crypted = data.substring(0, endCrypted);
		   		var beforeParts=this.receivedData.join('');
		   		var decrypted = segodoc.decrypt(beforeParts+crypted,request.URI.asciiSpec);
				var secondPart = data.substring(endCrypted + 16, data.length)
				data = decrypted + secondPart;
				SegodocUtils.log("RECONSTRUCTED: "+data);
				this.isPending=false;
		  	 }catch(e){
		  		alert(e);
		  	 }
				
		   } else if(data.indexOf("123456789crypted")!=-1) {
		   	SegodocUtils.log("FRAG BEGIN: "+data);
		   		var beginCrypted = data.indexOf("123456789crypted");
				//var endCrypted = data.indexOf("987654321crypted");
		   		var crypted = data.substring(beginCrypted+16, data.length);
		   		data=data.substring(0,beginCrypted);
		   		this.isPending=true;
		   		this.receivedData.push(crypted);				
				
		   } else {		   	
		   		if(this.receivedData.length>0 && this.isPending) {
		   			SegodocUtils.log("FRAG MID: "+data);
		   			this.receivedData.push(data);
		   			data="";		   			
		   		}	   
		   }
         }
		
		try {
           binaryOutputStream.writeBytes(data, data.length);
           inputStream = storageStream.newInputStream(0);
		} catch(e) {
		   SegodocUtils.log("Exception "+e+"\n"+data+"\n"+data.length);
		}

        this.originalListener.onDataAvailable(request, context, inputStream, offset, count);
    },

    onStartRequest: function(request, context) {
    	this.receivedData=new Array();
    	this.i=0;
    	this.originalListener.onStartRequest(request, context);
    },

    onStopRequest: function(request, context, statusCode)
    {
        this.originalListener.onStopRequest(request, context, statusCode);
    },

    QueryInterface: function (aIID) {
        if (aIID.equals(Components.interfaces.nsIStreamListener) ||
            aIID.equals(Components.interfaces.nsISupports)) {
            return this;
        }
        throw Components.results.NS_NOINTERFACE;
    }
}
