// **** BEGIN LICENSE BLOCK ****
// Copyright(c) 2008 Edoardo Gargano
//
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// Portions of this code have been based upon
// LiveHttpHeaders - http://livehttpheaders.mozdev.org
// Copyright(c) 2002-2003 Daniel Savard.
// And TamperData - http://tamperdata.mozdev.org
// Copyright(c) 2005 Adam Judson
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//
//
//  requestResponse.js - manage the http request and response
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// Request/Response object
function RequestResponse() {
}

RequestResponse.prototype = {
   // request stuff
   time: null,
   millis : null,
   onLoadMillis : null,
   uri : null,
   requestMethod : "?",   
   requestHeaders : null,
   postBodyHeaders : null,
   // this is a string
   requestPostData : null,
   requestPostDataIsBinary : false,
   // this isn't
   requestPostDataArray : null,

   elapsedTime : 0,  
   totalTime : 0,
   responseHeaders : null, 
   loadFlags : null,
   status : "pending",
   statusText : "",
   contentSize : "unknown",
   mimeType : "unknown",
   cacheClient : "HTTP",
   cacheData : null,

   cancel : function() {
      this.elapsedTime = "n/a";
      this.status = "cancelled";
      this.statusText = "cancelled";
      this.responseHeaders = new Array();
   },

   setLoadFromCache : function() {
      this.elapsedTime = "0";
      this.status = this.STATUS_FROM_CACHE;
      this.statusText = this.STATUS_FROM_CACHE;
      this.contentSize = "-1";
      this.mimeType = "unknown";
      this.responseHeaders = new Array();
   },

   setRequestData : function(uri, time, requestMethod, requestHeaders, postBodyHeaders, postData, loadFlags) {
      this.uri = uri;
      this.time = SegodocUtils.getTime(time);
      this.millis = time.getTime();
      this.loadFlags = loadFlags;
      this.requestMethod = requestMethod;
      this.requestHeaders = requestHeaders;
      if (postData) {
         this.requestPostData = postData.body;
         this.requestPostDataIsBinary = postData.binary;
         this.postBodyHeaders = postBodyHeaders;
      }
   },

   getPostData : function() {
      return this.requestPostData;
   },

   getPostBodyHeaders : function() {
      return this.postBodyHeaders;
   },

   // used for replaying the URI
   getPostDataStream : function() {
      // do something different if we are binary?
      if (this.requestPostData != null) {
         const MIME_STREAM_CID = "@mozilla.org/network/mime-input-stream;1";
         const nsIMIMEInputStream = Components.interfaces.nsIMIMEInputStream;
         var mis = Components.classes[MIME_STREAM_CID];
         post = mis.createInstance(nsIMIMEInputStream);
         post.setData(SegodocUtils.createStringInputStream(this.requestPostData));

         return post;
      } else {
         return null;
      }
   }

};
